import csv
from pathlib import PurePath
from typing import Dict

from ucp9.environment import ORIGINAL, TRANSCODED, UTF8


class _CP932Conversion:
    """
    A SingleTon class which provides transcoding service to cp932-incompatible characters
    """
    instance = None

    def __init__(self) -> None:
        self._character_dict: Dict[str, str] = self._dict_data_reader()

    def convert(self, char: str) -> str:
        """
        :param char: A single Japanese character
        :return: A cp932-friendly version of the given character
        """
        return self._character_dict[char]

    @staticmethod
    def _dict_data_reader() -> Dict[str, str]:
        conf_file_path = str(PurePath(str(PurePath(__file__).parent), "cp932.conf"))
        character_dict: Dict[str, str] = {}
        with open(conf_file_path, 'r', encoding=UTF8) as f:
            reader = csv.DictReader(f)
            for row in reader:
                character_dict[row[ORIGINAL]] = row[TRANSCODED]

        return character_dict


def CP932Conversion() -> _CP932Conversion:
    if _CP932Conversion.instance is None:
        _CP932Conversion.instance = _CP932Conversion()

    return _CP932Conversion.instance



from enum import Enum

from ucp9.conversion import CP932Conversion
from ucp9.environment import CP932


class Option(Enum):
    REMOVE = 'remove'
    KEEP = 'keep'
    REPLACE = 'replace'


def convert(string: str, option: str = 'replace') -> str:
    """
    Iterate through each character in the input string,
    attempt to convert any characters which cannot be encoded to cp932.
    :param string: a string of text.
    :param option: keep/replace/remove = action to be performed on inconvertible characters. Default is replace
    :return: a cp932-encodable version of the input string.
    """
    # Convert string to enum
    option = Option(option)
    output = ""
    if type(string) is not str:
        return string

    for c in string:
        try:
            c.encode(CP932)
            output += c
        except UnicodeEncodeError:
            output += _replacer(c, option)

    return output


def _replacer(character: str, option: Option) -> str:
    """
    :param character: a single character
    :return: cp932-compatible version of the input character
    """
    try:
        cp = CP932Conversion()
        return cp.convert(character)
    except KeyError:
        if option == Option.REMOVE:
            return ""
        elif option == Option.KEEP:
            return character
        else:
            return "?"



from ucp9.main import convert

"""
[C9CONV]

This module provides a transcoding service:
- FROM: An arbitrary unicode character.
- TO: A cp932-compatible, semantically similar but differently encoded version of the same character.

Author:
    Tran Anh Nhan
    
License:
    MIT License
    
Usage:
    import ucp9
    ucp9.convert(string, option)
        [string]: a string that contains cp932-incompatible characters
        [option]:
            - "keep": keep the cp932-inconvertible characters. NOTE: the return string WON'T be cp932-compatible.
            - "remove": remove the cp932-inconvertible characters.
            - "replace": replace the cp932-inconvertible characters with "?"
"""

convert = convert



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="ucp9",
    version="1.0.1",
    author="Tran Anh Nhan",
    author_email="vfa.nhanta@gmail.com",
    description="Unicode To CP932 Transcoder",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/nhantranz/ucp9",
    packages=["ucp9"],
    package_data={'': ['cp932.conf']},
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)
